@@ -3,7 +3,7 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 use ews::delete_folder::DeleteFolder;
-use std::marker::PhantomData;
+use std::{marker::PhantomData, sync::Arc};
 
 use super::{DoEraseFolder, DoOperation, XpComEwsClient};
 
@@ -11,7 +11,7 @@ use crate::{client::ServerType, safe_xpcom::SafeEwsSimpleOperationListener};
 
 impl<ServerT: ServerType> XpComEwsClient<ServerT> {
     pub async fn delete_folder(
-        self,
+        self: Arc<XpComEwsClient<ServerT>>,
         listener: SafeEwsSimpleOperationListener,
         folder_ids: Vec<String>,
     ) {
