@@ -6,6 +6,7 @@
 
 #include "DOMIntersectionObserver.h"
 
+#include "NonCustomCSSPropertyId.h"
 #include "Units.h"
 #include "mozilla/PresShell.h"
 #include "mozilla/ScrollContainerFrame.h"
@@ -15,9 +16,10 @@
 #include "mozilla/dom/BrowserChild.h"
 #include "mozilla/dom/BrowsingContext.h"
 #include "mozilla/dom/DocumentInlines.h"
+#include "mozilla/dom/Element.h"
+#include "mozilla/dom/ElementInlines.h"
 #include "mozilla/dom/HTMLIFrameElement.h"
 #include "mozilla/dom/HTMLImageElement.h"
-#include "nsCSSPropertyID.h"
 #include "nsContainerFrame.h"
 #include "nsContentUtils.h"
 #include "nsIFrame.h"
@@ -208,13 +210,12 @@ static LengthPercentage PrefMargin(float aValue, bool aIsPercentage) {
                        : LengthPercentage::FromPixels(aValue);
 }
 
-IntersectionObserverMargin DOMIntersectionObserver::LazyLoadingRootMargin() {
+static IntersectionObserverMargin LazyLoadingMargin() {
   IntersectionObserverMargin margin;
-#define SET_MARGIN(side_, side_lower_)                                 \
-  margin.Get(eSide##side_) = PrefMargin(                               \
-      StaticPrefs::dom_image_lazy_loading_root_margin_##side_lower_(), \
-      StaticPrefs::                                                    \
-          dom_image_lazy_loading_root_margin_##side_lower_##_percentage());
+#define SET_MARGIN(side_, side_lower_)                      \
+  margin.Get(eSide##side_) = PrefMargin(                    \
+      StaticPrefs::dom_lazy_loading_margin_##side_lower_(), \
+      StaticPrefs::dom_lazy_loading_margin_##side_lower_##_percentage());
   SET_MARGIN(Top, top);
   SET_MARGIN(Right, right);
   SET_MARGIN(Bottom, bottom);
@@ -234,7 +235,10 @@ DOMIntersectionObserver::CreateLazyLoadObserver(Document& aDocument) {
   RefPtr<DOMIntersectionObserver> observer =
       new DOMIntersectionObserver(aDocument, LazyLoadCallback);
   observer->mThresholds.AppendElement(0.0f);
-  observer->mRootMargin = LazyLoadingRootMargin();
+  auto* margin = StaticPrefs::dom_lazy_loading_margin_is_scroll()
+                     ? &observer->mScrollMargin
+                     : &observer->mRootMargin;
+  *margin = LazyLoadingMargin();
   return observer.forget();
 }
 
@@ -380,7 +384,7 @@ enum class BrowsingContextOrigin { Similar, Different };
 
 // NOTE(emilio): Checking docgroup as per discussion in:
 // https://github.com/w3c/IntersectionObserver/issues/161
-static BrowsingContextOrigin SimilarOrigin(const Element& aTarget,
+static BrowsingContextOrigin SimilarOrigin(const nsIContent& aTarget,
                                            const nsINode* aRoot) {
   if (!aRoot) {
     return BrowsingContextOrigin::Different;
@@ -429,8 +433,8 @@ static Maybe<nsRect> ComputeTheIntersection(
     nsIFrame* aRoot, const nsRect& aRootBounds,
     const IntersectionObserverMargin& aScrollMargin,
     const Maybe<nsRect>& aRemoteDocumentVisibleRect,
-    DOMIntersectionObserver::IsForProximityToViewport
-        aIsForProximityToViewport) {
+    DOMIntersectionObserver::IsForProximityToViewport aIsForProximityToViewport,
+    bool* aPreservesAxisAlignedRectangles) {
   nsIFrame* target = aTarget;
   // 1. Let intersectionRect be the result of running the
   // getBoundingClientRect() algorithm on the target.
@@ -472,9 +476,14 @@ static Maybe<nsRect> ComputeTheIntersection(
       // CrossDocParentFrame.
 
       // 3.2 Map intersectionRect to the coordinate space of container.
+      bool preservesAxisAlignedRectangles = false;
       nsRect intersectionRectRelativeToContainer =
           nsLayoutUtils::TransformFrameRectToAncestor(
-              target, intersectionRect.value(), containerFrame);
+              target, intersectionRect.value(), containerFrame,
+              &preservesAxisAlignedRectangles);
+      if (aPreservesAxisAlignedRectangles) {
+        *aPreservesAxisAlignedRectangles |= preservesAxisAlignedRectangles;
+      }
 
       // 3.3 If container is a scroll container, apply the
       // IntersectionObserver’s [[scrollMargin]] to the container’s clip rect as
@@ -497,9 +506,14 @@ static Maybe<nsRect> ComputeTheIntersection(
       // 3.4 TODO: Apply clip-path.
       if (!clipAxes.isEmpty()) {
         // 3.2 Map intersectionRect to the coordinate space of container.
+        bool preservesAxisAlignedRectangles = false;
         const nsRect intersectionRectRelativeToContainer =
             nsLayoutUtils::TransformFrameRectToAncestor(
-                target, intersectionRect.value(), containerFrame);
+                target, intersectionRect.value(), containerFrame,
+                &preservesAxisAlignedRectangles);
+        if (aPreservesAxisAlignedRectangles) {
+          *aPreservesAxisAlignedRectangles |= preservesAxisAlignedRectangles;
+        }
         const nsRect clipRect = OverflowAreas::GetOverflowClipRect(
             intersectionRectRelativeToContainer,
             containerFrame->GetRectRelativeToSelf(), clipAxes,
@@ -522,10 +536,15 @@ static Maybe<nsRect> ComputeTheIntersection(
   MOZ_ASSERT(intersectionRect);
 
   // 4. Map intersectionRect to the coordinate space of the intersection root.
+  bool preservesAxisAlignedRectangles = false;
   nsRect intersectionRectRelativeToRoot =
       nsLayoutUtils::TransformFrameRectToAncestor(
           target, intersectionRect.value(),
-          nsLayoutUtils::GetContainingBlockForClientRect(aRoot));
+          nsLayoutUtils::GetContainingBlockForClientRect(aRoot),
+          &preservesAxisAlignedRectangles);
+  if (aPreservesAxisAlignedRectangles) {
+    *aPreservesAxisAlignedRectangles |= preservesAxisAlignedRectangles;
+  }
 
   // 5.Update intersectionRect by intersecting it with the root intersection
   // rectangle.
@@ -539,9 +558,10 @@ static Maybe<nsRect> ComputeTheIntersection(
 
     intersectionRect = intersectionRectRelativeToRoot.EdgeInclusiveIntersection(
         *aRemoteDocumentVisibleRect);
-    if (intersectionRect.isNothing()) {
-      return Nothing();
-    }
+  } else if (aTarget->HasAnyStateBits(NS_FRAME_IN_POPUP)) {
+    // Popups don't get clipped to the viewport, so avoid applying the root
+    // intersection rect, see bug 1991410.
+    intersectionRect = Some(intersectionRectRelativeToRoot);
   } else {
     intersectionRect =
         intersectionRectRelativeToRoot.EdgeInclusiveIntersection(aRootBounds);
@@ -641,6 +661,16 @@ static Maybe<OopIframeMetrics> GetOopIframeMetrics(
   });
 }
 
+IntersectionInput DOMIntersectionObserver::ComputeInputForIframeThrottling(
+    const Document& aEmbedderDocument) {
+  auto margin = LazyLoadingMargin();
+  // TODO: Consider not using exactly the same parameters as lazy-load?
+  const bool useScroll = StaticPrefs::dom_lazy_loading_margin_is_scroll();
+  return ComputeInput(aEmbedderDocument, /* aRoot = */ nullptr,
+                      /* aRootMargin = */ useScroll ? nullptr : &margin,
+                      /* aScrollMargin = */ useScroll ? &margin : nullptr);
+}
+
 // https://w3c.github.io/IntersectionObserver/#update-intersection-observations-algo
 // step 2.1
 IntersectionInput DOMIntersectionObserver::ComputeInput(
@@ -734,15 +764,29 @@ IntersectionInput DOMIntersectionObserver::ComputeInput(
           remoteDocumentVisibleRect};
 }
 
-// https://w3c.github.io/IntersectionObserver/#update-intersection-observations-algo
-// (steps 2.1 - 2.5)
 IntersectionOutput DOMIntersectionObserver::Intersect(
     const IntersectionInput& aInput, const Element& aTarget, BoxToUse aBoxToUse,
     IsForProximityToViewport aIsForProximityToViewport) {
-  const bool isSimilarOrigin = SimilarOrigin(aTarget, aInput.mRootNode) ==
-                               BrowsingContextOrigin::Similar;
   nsIFrame* targetFrame = aTarget.GetPrimaryFrame();
-  if (!targetFrame || !aInput.mRootFrame) {
+  if (!targetFrame) {
+    return {SimilarOrigin(aTarget, aInput.mRootNode) ==
+            BrowsingContextOrigin::Similar};
+  }
+  return Intersect(aInput, targetFrame, aBoxToUse, aIsForProximityToViewport);
+}
+
+// https://w3c.github.io/IntersectionObserver/#update-intersection-observations-algo
+// (steps 2.1 - 2.5)
+IntersectionOutput DOMIntersectionObserver::Intersect(
+    const IntersectionInput& aInput, nsIFrame* aTargetFrame, BoxToUse aBoxToUse,
+    IsForProximityToViewport aIsForProximityToViewport) {
+  MOZ_ASSERT(aTargetFrame);
+
+  const nsIContent* target = aTargetFrame->GetContent();
+  const bool isSimilarOrigin =
+      target && SimilarOrigin(*target, aInput.mRootNode) ==
+                    BrowsingContextOrigin::Similar;
+  if (!aInput.mRootFrame) {
     return {isSimilarOrigin};
   }
 
@@ -756,14 +800,14 @@ IntersectionOutput DOMIntersectionObserver::Intersect(
   // specifically to determine the proximity to the viewport for
   // `content-visibility: auto` elements.
   if (aIsForProximityToViewport == IsForProximityToViewport::No &&
-      targetFrame->IsHiddenByContentVisibilityOnAnyAncestor()) {
+      aTargetFrame->IsHiddenByContentVisibilityOnAnyAncestor()) {
     return {isSimilarOrigin};
   }
 
   // 2.2. If the intersection root is not the implicit root, and target is
   // not in the same Document as the intersection root, skip to step 11.
   if (!aInput.mIsImplicitRoot &&
-      aInput.mRootNode->OwnerDoc() != aTarget.OwnerDoc()) {
+      aInput.mRootNode->OwnerDoc() != target->OwnerDoc()) {
     return {isSimilarOrigin};
   }
 
@@ -773,9 +817,9 @@ IntersectionOutput DOMIntersectionObserver::Intersect(
   // NOTE(emilio): We also do this if target is the implicit root, pending
   // clarification in
   // https://github.com/w3c/IntersectionObserver/issues/456.
-  if (aInput.mRootFrame == targetFrame ||
+  if (aInput.mRootFrame == aTargetFrame ||
       !nsLayoutUtils::IsAncestorFrameCrossDocInProcess(aInput.mRootFrame,
-                                                       targetFrame)) {
+                                                       aTargetFrame)) {
     return {isSimilarOrigin};
   }
 
@@ -799,21 +843,22 @@ IntersectionOutput DOMIntersectionObserver::Intersect(
     flags += nsLayoutUtils::GetAllInFlowRectsFlag::UseContentBox;
   }
   nsRect targetRectRelativeToTarget =
-      nsLayoutUtils::GetAllInFlowRectsUnion(targetFrame, targetFrame, flags);
+      nsLayoutUtils::GetAllInFlowRectsUnion(aTargetFrame, aTargetFrame, flags);
 
   if (aBoxToUse == BoxToUse::OverflowClip) {
-    const auto& disp = *targetFrame->StyleDisplay();
-    auto clipAxes = targetFrame->ShouldApplyOverflowClipping(&disp);
+    const auto& disp = *aTargetFrame->StyleDisplay();
+    auto clipAxes = aTargetFrame->ShouldApplyOverflowClipping(&disp);
     if (!clipAxes.isEmpty()) {
       targetRectRelativeToTarget = OverflowAreas::GetOverflowClipRect(
           targetRectRelativeToTarget, targetRectRelativeToTarget, clipAxes,
-          targetFrame->OverflowClipMargin(clipAxes));
+          aTargetFrame->OverflowClipMargin(clipAxes,
+                                           /* aAllowNegative = */ false));
     }
   }
 
   auto targetRect = nsLayoutUtils::TransformFrameRectToAncestor(
-      targetFrame, targetRectRelativeToTarget,
-      nsLayoutUtils::GetContainingBlockForClientRect(targetFrame));
+      aTargetFrame, targetRectRelativeToTarget,
+      nsLayoutUtils::GetContainingBlockForClientRect(aTargetFrame));
 
   // For content-visibility, we need to observe the overflow clip edge,
   // https://drafts.csswg.org/css-contain-2/#close-to-the-viewport
@@ -822,12 +867,14 @@ IntersectionOutput DOMIntersectionObserver::Intersect(
 
   // 2.5. Let intersectionRect be the result of running the compute the
   // intersection algorithm on target and observer’s intersection root.
+  bool preservesAxisAlignedRectangles = false;
   Maybe<nsRect> intersectionRect = ComputeTheIntersection(
-      targetFrame, targetRectRelativeToTarget, aInput.mRootFrame, rootBounds,
+      aTargetFrame, targetRectRelativeToTarget, aInput.mRootFrame, rootBounds,
       aInput.mScrollMargin, aInput.mRemoteDocumentVisibleRect,
-      aIsForProximityToViewport);
+      aIsForProximityToViewport, &preservesAxisAlignedRectangles);
 
-  return {isSimilarOrigin, rootBounds, targetRect, intersectionRect};
+  return {isSimilarOrigin, rootBounds, targetRect, intersectionRect,
+          preservesAxisAlignedRectangles};
 }
 
 IntersectionOutput DOMIntersectionObserver::Intersect(
@@ -840,7 +887,8 @@ IntersectionOutput DOMIntersectionObserver::Intersect(
     intersectionRect = intersectionRect->EdgeInclusiveIntersection(
         *aInput.mRemoteDocumentVisibleRect);
   }
-  return {true, rootBounds, aTargetRect, intersectionRect};
+  return {true, rootBounds, aTargetRect, intersectionRect,
+          /* mPreserverAxisAlignedRectangles= */ false};
 }
 
 // https://w3c.github.io/IntersectionObserver/#update-intersection-observations-algo
